<?php
/**
 * Template Name: Search & Jobs page
 *
 * This is the most generic template file in a WordPress theme
 * and one of the two required files for a theme (the other being style.css).
 * It is used to display a page when nothing more specific matches a query.
 * E.g., it puts together the home page when no home.php file exists.
 *
 * @link https://codex.wordpress.org/Template_Hierarchy
 *
 * @package WordPress
 * @subpackage Biuro
 * @since 1.0
 * @version 1.0
 */

get_header(); ?>

    <div class="l-section  l-section--search-page ">
        <div class="l-section--inner">
<?php
                get_search_form();
?>
        </div>
    </div>

    <div class="c-search--filters">
        <button class="js-toggle--aside o-btn c-btn--filter">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 1000 1000" width="17" height="17" class="c-ico--filter">
                <path d="M907.8,100.2l-346,380v419.7l-124.5-98.3v-322l-345-379.4H907.8 M907.8,17.9H92.2c-32.6,0-62,19.2-75.2,49c-13.2,29.8-7.5,64.5,14.4,88.6L355,511.3v290.2c0,25.2,11.5,49,31.3,64.6l124.5,98.3c14.8,11.7,32.8,17.7,51,17.7c12.2,0,24.5-2.7,35.9-8.2c28.4-13.7,46.4-42.5,46.4-74V512l324.6-356.5c21.9-24.1,27.6-58.8,14.4-88.6C969.8,37.1,940.3,17.9,907.8,17.9L907.8,17.9z"/>
            </svg> <?php _e('Other filters', 'biuro'); ?>
        </button>
    </div>

    <div id="content" class="l-main">

    <div class="l-content l-content--heading">
<?php

        if ( !is_search() ):
            while ( have_posts() ) :
                the_post();
                the_content();
            endwhile;
        endif;
 ?>
    </div>
    <div class="l-content">

        <aside class="l-aside l-aside--search">
<?php
            get_template_part( 'template-parts/layout/aside', 'job-list' );
?>
            <span class="js-toggle--aside l-aside--close">
                <svg width="13px" height="13px">
                    <use xlink:href="#ico--close"></use>
                </svg>
            </span>
        </aside><!-- .l-aside -->



        <main id="main" class="l-main">

<?php
    global $query_string;

    $keys =  get_post_custom_keys();

    $where = 'valid.meta_value > "' . date('Y-m-d', strtotime('-1 days')) . '"';

    if ( is_search() ):

        wp_parse_str( $query_string, $searchQuery );

        if ( $searchQuery ) :

            if ( $searchQuery['city'] ) :
                $city = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['city']), 'city' );

                if ( $city ) :
                    $where = $where . ' AND city.term_id = ' . $city->term_id;
                endif;
            endif;

            if ( $searchQuery['s'] ) :

                $field = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['s']), 'field' );
                $type = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['s']), 'type' );

                if ( $field ) :
                    $where = $where . ' AND field.term_id = ' . $field->term_id;
                elseif ( $type ) :
                    $where = $where . ' AND type.term_id = ' . $type->term_id;
                else:
                    $where = $where . ' AND (t.post_title LIKE "%' . $searchQuery['s'] . '%" OR  t.post_content LIKE "%' . $searchQuery['s'] . '%")';
                endif;

            endif;
        endif;

        // $where = 'valid.meta_value > "' . date('Y-m-d', strtotime('-1 days')) . '" AND (t.post_title LIKE "%' . $searchQuery . '%" OR  t.post_content LIKE "%' . $searchQuery . '%")';

    elseif ($keys):
        foreach ( $keys as $key => $value ) :
            $val = trim($value);
            $taxonomy = substr($val, 20);

            if ( '_' == $val{0} || substr($val, 0, 20) != 'built_in_taxonomies_' || !$taxonomy):
                continue;
            endif;

            $res = get_post_custom_values( $val );

            if ( !$res ) :
                continue;
            endif;

            $term = $res[0];

            if ( !$term ) :
                continue;
            endif;

            $where = $where . ' AND ' .  $taxonomy . '.slug = "' . $term . '"';
        endforeach;
    endif;

    // WP Pods fix for multisite
    global $wpdb;

    $params = array(
        'orderby' => 'date DESC',
        'where' => $where,
        'limit'   => 20
    );

    if (get_current_blog_id() != 1):
        $langs = pll_the_languages(array('raw'=>1));
        $langID = $langs[pll_current_language('slug')]['id'];
        $where = $where . ' AND ( `langs`.`object_id` IS NOT NULL )';

        $params['join'] = ' LEFT JOIN `' . $wpdb->prefix . 'term_relationships` AS `langs` ON `langs`.`object_id` = `t`.`ID` AND `langs`.`term_taxonomy_id` = ' . $langID;
        $params['where'] = $where;
    endif;
    // END WP Pods fix for multisite

    $jobs = pods( 'job', $params );

    if ( 0 < $jobs->total() ):

        get_template_part( 'template-parts/jobs/jobs', 'list' );

    else:

        get_template_part( 'template-parts/jobs/jobs', 'not-found' );

    endif;
?>

            </main><!-- .l-main -->
        </div><!-- .l-content -->

    </div><!-- #content -->

<?php get_footer();
