<?php
/**
 * The main template file
 *
 * This is the most generic template file in a WordPress theme
 * and one of the two required files for a theme (the other being style.css).
 * It is used to display a page when nothing more specific matches a query.
 * E.g., it puts together the home page when no home.php file exists.
 *
 * @link https://codex.wordpress.org/Template_Hierarchy
 *
 * @package WordPress
 * @subpackage Biuro
 * @since 1.0
 * @version 1.0
 */

get_header();
?>
<div id="content" class="l-content--position">

    <div class="l-content--position-inner">

        <main id="main" class="l-main l-main--position">

<?php
        $ID = get_the_id();

        while ( have_posts() ) :
?>
            <ul class="c-breadcrumbs">
                <li>
                    <a href="<?php echo pll_home_url(); ?>" ><?php _e('Main page', 'biuro'); ?></a>
                </li>
                <li class="c-breadcrumb--sep">
                    <svg width="4px" height="9.376px" class="c-ico--right">
                        <use xlink:href="#ico--right-slim"></use>
                    </svg>
                </li>
<?php
            global $positionsPage;

            if ($positionsPage):
?>
                <li>
                    <a href="<?php echo $positionsPage; ?>"><?php _e('Job offers', 'biuro'); ?></a>
                </li>
                <li class="c-breadcrumb--sep">
                    <svg width="4px" height="9.376px" class="c-ico--right">
                        <use xlink:href="#ico--right-slim"></use>
                    </svg>
                </li>
<?php
            endif;
?>
                <li class="c-breadcrumb--active">
                    <a href="<?php echo the_permalink(); ?>"><?php the_title(); ?></a>
                </li>
            </ul>

            <h1 class="c-job--title"><?php the_title(); ?></h1>

<?php

    function getCategory($ID, $str) {
        $res = array(
            'name' => '',
            'pageURL' => '',
        );

        $terms = get_post_meta($ID, "_yoast_wpseo_primary_{$str}");

        if (isset($terms) && isset($terms[0])) {
            $pages = getOptions($str);
            $termID = $terms[0];

            $options = array_filter($pages, function ($pages) use ($termID) { return ($pages['termID'] == $termID); } );

            foreach ($options as &$page) :
                if (isset($page['name'])):
                    $res['name'] = $page['name'];
                endif;

                if (isset($page['pageID'])):
                    $res['pageURL'] = get_page_link( $page['pageID'] );
                else:
                    $res['pageURL'] = ($str == 'city') ? '/?city=' . $res['name'] . '&s=' : '/?city=&s=' . $res['name'];
                endif;
            endforeach;
        }

        return $res;
    }


    // delog('type');
    // $fields = get_post_meta($ID, '_yoast_wpseo_primary_type');
    // debug($fields);

?>

            <ul class="c-categories">
<?php
            $city = getCategory($ID, 'city');

            if ($city['name'] != ''):
?>
                <li>
                    <a href="<?php echo $city['pageURL']; ?>" ><?php echo $city['name']; ?></a>
                </li>
<?php
            endif;

            $field = getCategory($ID, 'field');

            if ($field['name'] != ''):
                if ($city['name'] != ''):
?>
                <li class="c-categories--sep">/</li>
<?php
                endif;
?>
                <li>
                    <a href="<?php echo $field['pageURL']; ?>" ><?php echo $field['name']; ?></a>
                </li>
<?php
            endif;

            $types = wp_get_post_terms( $ID, 'type', array('orderby' => 'count', 'order' => 'DESC'));
            $options = getOptions('type');

            usort($options, function($a, $b) {
                return $a['no'] < $b['no'];
            });

            $type = array(
                'name' => '',
                'pageURL' => '',
            );

            foreach ($options as &$option) :
                foreach ($types as &$page) :
                    if ($page->term_id == $option['termID']):
                        if (isset($page->name)):
                            $type['name'] = $page->name;
                        endif;

                        if (isset($option['pageID'])):
                            $type['pageURL'] = get_page_link( $option['pageID'] );
                        else:
                            $type['pageURL'] = '/?city=&s=' . $type['name'];
                        endif;

                        break 2;
                    endif;

                endforeach;
            endforeach;

            if ($type['name'] != ''):
                if ($field['name'] != '' || $city['name'] != ''):
?>
                <li class="c-categories--sep">/</li>
<?php
                endif;
?>
                <li>
                    <a href="<?php echo $type['pageURL']; ?>" ><?php echo $type['name']; ?></a>
                </li>
<?php
            endif;
?>
            </ul>

            <div class="c-job--content">
<?php
                the_post();
                the_content();
?>
            </div>
<?php
        endwhile;

        $pod = pods( 'job', get_the_id() );

        if (!is_array($pod->field( 'salary_from' )) && $pod->field( 'salary_from' ) > 0 || !is_array($pod->field( 'salary_to' )) && $pod->field( 'salary_to' ) > 0):
?>
            <p>
<?php
            if ($pod->field( 'salary_from' ) > 0):
?>
                <?php _e('Salary from', 'biuro'); ?>: <?php echo $pod->field( 'salary_from' ); ?>
                <br>
<?php
            endif;
            if ($pod->field( 'salary_to' ) > 0):
?>
                <?php _e('Salary to', 'biuro'); ?>: <?php echo $pod->field( 'salary_to' ); ?>
<?php
            endif;
?>

            </p>
<?php
        endif;

        $validPosition = date_i18n( 'Y-m-d', strtotime( $pod->field( 'valid' ) ) ) >= date('Y-m-d') ? true : false;
?>
            <p>
                <?php _e('Published', 'biuro'); ?>: <?php echo date_i18n( get_option( 'date_format' ), strtotime( $pod->field( 'post_date' ) ) ); ?>
                <br>
                <?php _e('Valid till', 'biuro'); ?>: <?php echo date_i18n( get_option( 'date_format' ), strtotime( $pod->field( 'valid' ) ) ); ?>
            </p>

<?php
    if ($validPosition):
 ?>
            <div class="c-job--action">
                <button type="button" class="js-inititate-position-form o-btn c-btn--main c-btn--fill" name="submit" value="1" data-id="action--form"><?php _e('Apply', 'biuro'); ?></button>
            </div>
<?php
    endif;
 ?>
        </main><!-- .l-main -->

        <aside class="l-aside--position">
<?php
        if ($validPosition):
            get_template_part( 'template-parts/layout/aside', 'job' );
        else:
            get_template_part( 'template-parts/layout/aside', 'job-inactive' );
        endif;
?>
        </aside><!-- .l-aside -->

    </div><!-- .l-content--position-inner -->

</div><!-- #content -->

<?php get_footer();
